#
# The Weather Tag
#
# (C) Copyright 2000 The Internet (Aust) Pty Ltd
# ACN: 082 081 472	ABN: 83 082 081 472
# All Rights Reserved
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# Author: Andrew Milton <akm@theinternet.com.au>

from ftplib import FTP
import string
import sys,os
import re
import time


import App, Globals, OFS

from Persistence import Persistent
from Globals import HTMLFile, HTML, MessageDialog, package_home
from Acquisition import Implicit
from OFS.Folder import Folder
from App.Extensions import getObject
from webdav.NullResource import NullResource
from ZPublisher.HTTPResponse import HTTPResponse


manage_addWeatherForm=HTMLFile('manage_addWeatherForm', globals())

def manage_addWeather(self, id, title, ICAO, ZuluDistance=0, REQUEST={}):
	""" Add a new Weather Site """

	obj=Weather(id=id, ICAO=ICAO, title=title, ZuluDistance=ZuluDistance)
	self._setObject(id, obj)
	obj=getattr(self, id)

	if hasattr(obj, 'postInitialisation'):
		obj.postInitialisation()

	if REQUEST:
		return self.manage_main(self, REQUEST)
	return ''	

class Weather(Folder, Persistent, Implicit):

	icon='misc_/ZWeather/Weather.gif'
	disabled=0
	lastStatus=0
	lastFail=0

	meta_type='Weather'

	manage_editWeatherForm=HTMLFile('manage_editWeatherForm', globals())
	
	manage_options=(
		{'label':'Manage Weather', 'action':'manage_editWeatherForm'},
		{'label':'Contents', 'action':'manage_main'},
		{'label':'View', 'action':'index_html'},
		{'label':'About', 'action':'about'},
		)

	def __init__(self, id, ICAO, title, ZuluDistance=0):
		self.id=id
		self.ICAO=ICAO
		self.title=title
		self.ZuluDistance=ZuluDistance # Used to determine night/day
		
		self.SkyConditions=''
		self.Temperature=''
		self.DewPoint=''
		self.Pressure=''
		self.Humidity=''
		self.Weather=''
		self.Visibility=''
		self.Direction=''
		self.WindSpeed=''
		self.WindChill=''

	def manage_editWeather(self, ICAO, title, ZuluDistance, enable=0,
						   REQUEST={}):
		""" Edit Weather Properties """
		self.ICAO=ICAO
		self.title=title
		self.name=title
		self.ZuluDistance=ZuluDistance
		if enable:
			self.disable=0

		if REQUEST:
			return self.manage_main(self, REQUEST)
		return ''

	# Present a uniform interface for getting attributes whether or
	# not metric conversion is required.
	def getSkyConditions(self, metric=''):
		""" Sky Conditions """
		return self.SkyConditions

	def getTemperature(self, metric=''):
		""" Current Temperature """
		if metric and self.Temperature!='':
			return str(int((int(self.Temperature)-32)*5.0/9.0)) # C
		return self.Temperature

	def getDewPoint(self, metric=''):
		""" Dew Point """
		if metric and self.DewPoint !='':
			return str(int((int(self.DewPoint)-32)*5.0/9.0)) # C
		return self.DewPoint

	def getPressure(self, metric=''):
		""" Air Pressure """
		if metric and self.Pressure!='':
			return str(int(float(self.Pressure)*3.38639)) # kPa
		return self.Pressure

	def getWindSpeed(self, metric=''):
		""" Wind Speed """
		if metric and self.WindSpeed!='':
			return str(int(float(self.WindSpeed)*1.609)) # km/h
		self.WindSpeed

	def getWeather(self, metric=''):
		""" Other Weather Conditions """
		return self.Weather

	def getHumidity(self, metric=''):
		""" Get Humidity """
		return self.Humidity

	def getVisibility(self, metric=''):
		""" Visibility """
		return self.Visibility

	def getDirection(self, metric=''):
		""" Direction """
		return self.Direction

	def getWindChill(self, metric=''):
		""" Wind Chill """
		if metric and self.WindChill!='':
			return str(int((int(self.WindChill)-32)*5.0/9.0)) # C
		return self.WindChill
		
	def postInitialisation(self):
		self.fetchWeather()

	def index_html(self, metric=0, REQUEST={}):
		""" The Index File """
		if not self.disabled:
			if (self.lastFetched + int(self.freq)) < time.time():
				try:
					self.fetchWeather()
					self.lastFetched=time.time()
					self.lastStatus=1
					self.disabled=0
				except:
					self.lastStatus=0
					self.lastFail=self.lastFail+1
					if self.lastFail == 3:
						self.disabled=1
			self._p_changed=1
			return self.showWeather(self, REQUEST, metric=metric)

	def getLocalTime(self):
		updateTime=int(string.split(self.codedMETAR)[1][2:-1])

		# Zulu Time + Offset == Local Time
		updateTime+=int(float(self.ZuluDistance)*100.0)

		if updateTime < 0:
			updateTime+=2400
		if updateTime >= 2400:
			updateTime%=2400

		return updateTime

	def isNightTime(self):
		localTime=self.getLocalTime()
		# Let's be arbitrary...
		# Morning starts at 0600 and Night starts at 1800
		# It's just for the icons..
		return(localTime < 600 or localTime > 1800)

	def dayToDate(self, advanceBy):
		dt=DateTime.now() + DateTime.RelativeDateTime(days=advanceBy)	 
		today = DateTime.now()
		return str(dt)
	
	def chewLines(self, ftpLine):

		if re.search("^Sky conditions:",ftpLine) :
			self.SkyConditions=re.sub("Sky conditions: (.*)","\\1",ftpLine);
			return

		if re.search("^Temperature:",ftpLine) :
			self.Temperature=re.sub("Temperature:\s*(\-{0,1}[0-9.]{1,}).*","\\1",ftpLine)
			return

		if re.search("^Windchill:",ftpLine) :
			self.WindChill=re.sub("Windchill:\s*(\-{0,1}[0-9.]{1,}).*","\\1",ftpLine)
			return

		if re.search("^Dew Point:",ftpLine) :
			self.DewPoint=re.sub("Dew Point:\s*(\-{0,1}[0-9.]{1,}).*","\\1",ftpLine)
			return

		if re.search("^Pressure (.*):",ftpLine) :
			self.Pressure=re.sub("Pressure \(.*\):\s*([0-9.]{2,}).*","\\1",ftpLine)
			return

		if re.search("^Relative Humidity:",ftpLine) :
			self.Humidity=re.sub("Relative Humidity:\s*(\d{1,})\%.*","\\1",ftpLine)
			return

		if re.search("^Weather:",ftpLine):
			self.Weather=re.sub("Weather: (.*)","\\1",ftpLine)
			return

		if re.search("^Visibility:",ftpLine):
			self.Visibility=re.sub("Visibility: (.*):.*","\\1",ftpLine)
			return
		
		if re.search("^ob:",ftpLine) :
			self.codedMETAR=re.sub("ob: (.*)","\\1",ftpLine)
			return
		return

	def fetchWeather(self):
		""" Fetch the Weather File """

		localWeather = {}
		varFlag=0
		windGroup=''
		varWindGroup=''
		gustFlag=0
		Direction=0
		Speed=0.0
		Speed1=0.0
		Speed2=0.0

		self.Temperature=''
		self.Visibility=''
		self.Weather=''
		self.Humidity=''
		self.Pressure=''
		self.DewPoint=''
		self.WindChill=''
		self.SkyConditions=''
		
		fileName=self.ICAO+'.TXT'
		
		ftp=FTP(self.host)
		userid = passwd = acct = ''
		ftp.login(userid,passwd,acct)
		ftp.sendcmd('CWD '+self.weatherDir)
		ftp.retrlines('RETR '+fileName, self.chewLines)
		ftp.quit()

		windGroup=self.codedMETAR
		windGroup=re.sub("RML\s.*R","",windGroup)
		varWindGroup=windGroup
		if re.search(".*\s\w{3}\d{2,3}KT\s\d{3}V\d{3}\s.*",windGroup) :
			varWindGroup=re.sub(".*\s\w{3}\d{2,3}KT\s(\d{3}V\d{3})\s.*","\\1",windGroup)
			varFlag=1
		elif re.search(".*\s\w{3}\d{2,3}MPS\s\d{3}V\d{3}\s.*",windGroup) :
			varWindGroup=re.sub(".*\s\w{3}\d{2,3}G\d{2,3}KT\s(\d{2,3}V\d{2,3})\s.*","\\1",windGroup)
			varFlag=1
		elif re.search(".*\s\w{3}\d{2,3}G\d{2,3}MPS\s(\d{2,3}V\d{2,3})\s.*",windGroup) :
			varWindGroup=re.sub(".*\s\w{3}\d{2,3}G\d{2,3}MPS\s(\d{2,3}V\d{2,3})\s.*",windGroup)
			varFlag=1
		else:
			varWindGroup=""
			varFlag=0

		Direction1 = re.sub("(\d{3})V\d{3}","\\1",varWindGroup)
		Direction2 = re.sub("\d{3}V(\d{3})","\\1",varWindGroup)

		if re.search(".*\s\w{3}\d{2,3}KT\s.*",windGroup) :
			windGroup=re.sub(".*\s(\w{3}\d{2,3}KT)\s.*","\\1",windGroup)
			Direction=int(re.sub("(\w{3})\d{2,3}KT","\\1",windGroup))
			Speed = float(re.sub("\w{3}(\d{2,3})KT","\\1",windGroup))
		elif re.search(".*\s\w{3}\d{2,3}MPS\s.*",windGroup) :
			windGroup=re.sub(".*\s(\w{3}\d{2,3}G\d{2,3}KT)\s.*","\\1",windGroup)
			Direction=int(re.sub("(\w{3})\d{2,3}G\d{2,3}KT","\\1",windGroup))
			Speed1=float(re.sub("\w{3}(\d{2,3})G\d{2,3}KT",windGroup))
			Speed2=float(re.sub("\w{3}\d{2,3}G(\d{2,3})KT",windGroup))
			gustFlag=1
		elif re.search(".*\s\w{3}\d{2,3}G\d{2,3}MPS\s.*",windGroup) :
			windGroup=re.sub(".*\s(\w{3}\d{2,3}G\d{2,3}MPS)\s.*","\\1",windGroup)
			Direction=int(re.sub("(\w{3})\d{2,3}G\d{2,3}MPS","\\1",windGroup))
			Speed1=float(re.sub("\w{3}(\d{2,3})G\d{2,3}MPS","\\1",windGroup))
			Speed2=float(re.sub("\w{3}\d{2,3}G(\d{2,3})MPS","\\1",windGroup))
			Speed1 = Speed1 * 1.942
			Speed2 = Speed2 * 1.942
			gustFlag=1
		else:
			windGroup=''
	
		if re.search(self.ICAO+" \d{2}\d{2}Z",self.codedMETAR) :
			universalTime=re.sub('.*'+self.ICAO+" (\d{2})(\d{2})Z .*","\\1:\\2",
								 self.codedMETAR)
		else:
			universalTime='99:99'

		self.StationInfo=self.ICAO
		self.UniversalTime=universalTime

		if(not Direction):
			self.Direction='-9999'
		elif (Direction == 'VRB'):
			self.Direction='999'
		elif (varFlag):
			self.Direction=-(Direction)
		else:
			self.Direction=Direction

		if(not windGroup):
			self.WindSpeed='-9999'
		elif (gustFlag):
			AvgSpeed = (Speed1 + Speed2) / 2.0
			AvgSpeed = AvgSpeed * 1.15155
			self.WindSpeed=str(-AvgSpeed)
		else:
			Speed = Speed * 1.5155
			self.WindSpeed=str(Speed)
	
		self.lastFetched=time.time()
		self._p_changed=1
