#
# The Weather Tag
#
# (C) Copyright 2000 The Internet (Aust) Pty Ltd
# ACN: 082 081 472	ABN: 83 082 081 472
# All Rights Reserved
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# Author: Andrew Milton <akm@theinternet.com.au>

import App, Globals, OFS
import string
import time

from Persistence import Persistent
from Globals import ImageFile, HTMLFile, HTML, MessageDialog, package_home
from Acquisition import Implicit
from OFS.Folder import Folder
from App.Extensions import getObject
from webdav.NullResource import NullResource
from ZPublisher.HTTPResponse import HTTPResponse

from urllib import quote

def manage_addWeatherManagement(self, id, title='', REQUEST={}):
		""" Add a new Weather Management System """

		obj=WeatherManagement(id, title)
		self._setObject(id, obj)
		obj=getattr(self, id)
		if hasattr(obj, 'postInitialisation'):
				obj.postInitialisation()

		if REQUEST:
				return self.manage_main(self, REQUEST)
		return ''

manage_addWeatherManagementForm=HTMLFile('manage_addWeatherManagementForm', globals())

from Weather import Weather, manage_addWeather, manage_addWeatherForm

class WeatherManagement(Folder, Persistent, Implicit):
	""" Manage your RSS Weathers """

	meta_type="Weather Management"

	icon='WeatherManagement.gif'
	
	manage_addWeatherForm = manage_addWeatherForm
	manage_addWeather	  = manage_addWeather
	manage_addDTMLMethod=OFS.DTMLMethod.addDTMLMethod
	maange_addFolderMethod=OFS.Folder.manage_addFolder
	
	about=HTMLFile('about', globals())
	manage_configureWeatherManagementForm=HTMLFile('manage_configureWeatherManagementForm',globals())

	freq = 3600 # seconds -- 60 mins
	
	manage_options=(
		{'label':'New Weather', 'action':'manage_addWeatherForm'},            
		{'label':'Contents', 'action':'manage_main'},
        {'label':'Configuration', 'action':'manage_configureWeatherManagementForm'},		
		{'label':'About', 'action':'about'},
		)

	__ac_permission__=(
		('Add Weather', ('manage_addWeatherForm', 'manage_addWeather',),),
        ('View management screens', ('manage_configureWeatherManagementForm', 'manage_configureWeatherManagement'),),
		)

	
	weatherDir='/data/observations/metar/decoded'
	host='weather.noaa.gov'

	def __init__(self, id, title):
		self.id=id
		self.title=title

	def manage_configureWeatherManagement(self, host, weatherDir, freq, REQUEST={}):
		""" Configure Weather Stuff """

		ifreq=int(freq)
		if ifreq < 30:
			ifreq =30

		self.freq=ifreq*60 # Go from minutes to seconds

		self.host=host
		self.weatherDir=weatherDir

		if REQUEST:
			return self.manage_main(self, REQUEST)
		return ''

	# Get One Weather
	def getWeather(self, weatherId):
		if hasattr(self, weatherId):
			weather=getattr(self, weatherId)
			return weather

	def weatherList(self):
		""" Return the Weather Items """
		return self.objectIds('Weather')
	

	def displayWeather(self, weatherId, metric=0, REQUEST={}):
		""" Display One Weather """
		if hasattr(self, weatherId):
			weather=getattr(self, weatherId)
			return weather.index_html(metric=metric, REQUEST=REQUEST)

	icons=[
			{'Sky':'Cloudy', 'Image':'Cloudy.gif'},
			{'Sky':'Fine', 'Image':'Fine.gif'},
			{'Sky':'Fog', 'Image':'Fog.gif'},
			{'Sky':'Mist', 'Image':'Mist.gif'},
			{'Sky':'MostlyClear', 'Image':'MostlyClear.gif'},
			{'Sky':'MostlyCloudy', 'Image':'MostlyCloudy.gif'},
			{'Sky':'Overcast', 'Image':'Overcast.gif'},			   
			{'Sky':'PartlyCloudy', 'Image':'PartlyCloudy.gif'},
			{'Sky':'Rain', 'Image':'Rain.gif'},
			{'Sky':'Showers', 'Image':'Showers.gif'},
			{'Sky':'Snow', 'Image':'Snow.gif'},
			{'Sky':'Thunder', 'Image':'Thunder.gif'},

			{'Sky':'Cloudy_Night', 'Image':'Cloudy_Night.gif'},
			{'Sky':'Fine_Night', 'Image':'Fine_Night.gif'},
			{'Sky':'Fog_Night', 'Image':'Fog_Night.gif'},
			{'Sky':'Mist_Night', 'Image':'Mist_Night.gif'},
			{'Sky':'MostlyClear_Night', 'Image':'MostlyClear_Night.gif'},
			{'Sky':'MostlyCloudy_Night', 'Image':'MostlyCloudy_Night.gif'},
			{'Sky':'Overcast_Night', 'Image':'Overcast_Night.gif'},			   
			{'Sky':'PartlyCloudy_Night', 'Image':'PartlyCloudy_Night.gif'},
			{'Sky':'Rain_Night', 'Image':'Rain_Night.gif'},
			{'Sky':'Showers_Night', 'Image':'Showers_Night.gif'},
			{'Sky':'Snow_Night', 'Image':'Snow_Night.gif'},
			{'Sky':'Thunder_Night', 'Image':'Thunder_Night.gif'},
            
		]
	
	def postInitialisation(self):
		self.base_url=self.absolute_url()
		self.manage_addDTMLMethod(id='showWeather', title='Weather Box', file=HTMLFile('showWeather', globals()))
		# self.manage_addFolder(id='icons', title='Weather Icons')
		for ic in self.icons:
			print "Adding %s as %s"%(ic['Sky'], ic['Image'])
			setattr(self, ic['Image'], ImageFile('icons/'+ic['Image'],globals()))
		self._p_changed=1

	def getIconNameFromWeather(self, weather, nightFlag=0):
		#
		# These are free form strings, we try to find the
		# most severe weather condition and use that
		#
		if string.find(weather, 'snow') >=0:
			if nightFlag:
				return getattr(self,'Snow_Night.gif')
			return getattr(self,'Snow.gif')
		if string.find(weather, 'thunder') >=0:
			if nightFlag:					 
				return getattr(self, 'Thunder_Night.gif')
			return getattr(self, 'Thunder.gif')
		if string.find(weather, 'rain') >=0:
			if nightFlag:
				return getattr(self, 'Rain_Night.gif')
			return getattr(self, 'Rain.gif')
		if string.find(weather, 'showers') >=0:
			if nightFlag:
				return getattr(self, 'Showers_Night.gif')
			return getattr(self, 'Showers.gif')
		if string.find(weather, 'fog') >=0:
			if nightFlag:
				return getattr(self, 'Fog_Night.gif')
			return getattr(self, 'Fog.gif')
		if string.find(weather, 'mist') >=0:
			if nightFlag:
				return getattr(self, 'Mist_Night.gif')
			return getattr(self, 'Mist.gif')
		return 'No Weather'
	
	def getIconName(self, SkyConditions, nightFlag=0):
		if nightFlag:
			return string.join(string.split(string.capwords(string.strip(SkyConditions)),' '),'')+'_Night.gif'					  
		return string.join(string.split(string.capwords(string.strip(SkyConditions)),' '),'')+'.gif'

	def WeatherIcon(self, skyConditions, weather, nightFlag):
		""" Return the Weather Icon """
		icon=self.getIconNameFromWeather(weather,nightFlag) # Try Weather first
		if hasattr(self, icon): 
			return "%s/%s"%(self.base_url,icon)
		
		icon=self.getIconName(skyConditions, nightFlag)
		if hasattr(self, icon):
			return "%s/%s"%(self.base_url,icon)				   

		#
		# Return Fine gif as a fallback
		print "Don't have [%s]"%(icon)
		if nightFlag:
			return "%s/%s"%(self.base_url,'Fine_Night.gif')
		return "%s/%s"%(self.base_url,'Fine.gif')
